
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#ifdef	__MSVCRT__
#define stat	_stati64
#define LLU	"%I64u"
#define LLD	"%I64d"
#else
#define LLU	"%llu"
#define LLD	"%lld"
#endif

static void	Error(char *msg)
{
	(void)fputs(msg,stderr); (void)fputc('\n',stderr); exit(1);
}

static int	byte=1,kilo=1,mega=1,giga=0,tera=0;

struct DS {
	unsigned long long	files,dirs,bytes;
};

static void	DirSize(long long level,char *Path,struct DS *pDS)
{
	DIR		*d;
	size_t		l;
	struct DS	ds;
	struct dirent	*e;
	char		*path;
	struct stat	s;

	if ((d=opendir(Path))==NULL) {
	    (void)fprintf(stderr,"%s : directory read error.\n",Path); return;
	}
	l=strlen(Path)+2;
	ds.files=ds.dirs=ds.bytes=0;
	while ((e=readdir(d))!=NULL) {
	    if (!strcmp(e->d_name,".") || !strcmp(e->d_name,"..")) continue;

	    if ((path=(char *)malloc(l+strlen(e->d_name)))==NULL)
		Error("memory allocation error.");

	    (void)sprintf(path,"%s/%s",Path,e->d_name);
	    if (stat(path,&s)==(-1))
		(void)fprintf(stderr,"%s : status read error.\n",path);
	    else
		if ((s.st_mode&S_IFMT)==S_IFDIR) {
		    DirSize(level-1,path,&ds); ds.dirs++;
                }
		else {
		    ds.files++; ds.bytes+=s.st_size;
		}

	    free(path);
	}
	(void)closedir(d);

#define KILO	((unsigned long long)1<<10)
#define MEGA	((unsigned long long)1<<20)
#define GIGA	((unsigned long long)1<<30)
#define TERA	((unsigned long long)1<<40)

	if (level>0) {
	    (void)printf("%s\t" LLU "\t" LLU,Path,ds.files,ds.dirs);

	    if (byte) (void)printf("\t" LLU,ds.bytes);
	    if (kilo) (void)printf("\t" LLU,(ds.bytes+KILO-1)/KILO);
	    if (mega) (void)printf("\t" LLU,(ds.bytes+MEGA-1)/MEGA);
	    if (giga) (void)printf("\t" LLU,(ds.bytes+GIGA-1)/GIGA);
	    if (tera) (void)printf("\t" LLU,(ds.bytes+TERA-1)/TERA);

	    (void)putchar('\n');
	}
	if (pDS!=NULL) {
	    pDS->files+=ds.files; pDS->dirs+=ds.dirs; pDS->bytes+=ds.bytes;
	}
}

int	main(int argc,char **argv)
{
	int		argi,paths=0,bkmgt=0;
	long long	level=(long long)((unsigned long long)(~0)>>1);

	for (argi=1; argi<argc; argi++)
	    if (argv[argi][0]!='-')
		++paths;
	    else
		switch(argv[argi][1]) {
		    case 'b':
			byte=1; if (++bkmgt==1) kilo=mega=giga=tera=0; break;
		    case 'k':
			kilo=1; if (++bkmgt==1) byte=mega=giga=tera=0; break;
		    case 'm':
			mega=1; if (++bkmgt==1) byte=kilo=giga=tera=0; break;
		    case 'g':
			giga=1; if (++bkmgt==1) byte=kilo=mega=tera=0; break;
		    case 't':
			tera=1; if (++bkmgt==1) byte=kilo=mega=giga=0; break;
		    default :
			if (sscanf(argv[argi]+1,LLD,&level)!=1 || level<=0)
			    Error(
	"usage : dirsize {-outputLevel} {-b} {-k} {-m} {-g} {-t} {directories}"
			    );

			break;
		}

	if (paths==0)
	    DirSize(level,".",NULL);
	else
	    for (argi=1; argi<argc; argi++)
		if (argv[argi][0]!='-') DirSize(level,argv[argi],NULL);

	return 0;
}
