
/*
	Lister for C
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static int	GetToken(fp,tlen,tbuf,chckc)
FILE		*fp;
int		tlen;
char		*tbuf;
int		(*chckc)();
{
	int	c,l;

	if ((c=fgetc(fp))==EOF) return(-1);
	if (!(*chckc)(*tbuf=c))
	    l=0;
	else {
	    for (l=2; l<tlen; l++)
		if ((*chckc)(c=fgetc(fp))) *(++tbuf)=c; else break;
	    if (l!=tlen) {
		ungetc(c,fp); --l;
	    }
	}
	*(++tbuf)=0; return(l);
}

static int	CheckChar(c)
int		c;
{
	if ((c>='a' && c<='z') || (c>='A' && c<='Z') ||
	   ((c>='0' && c<='9') || c=='_')) return(1); else return(0);
}

#define KEYWORD_LENGTH_MAX 32
#define DEFAULT_KEYWORD_NO 36

static char	*defaultKeyword[DEFAULT_KEYWORD_NO] = {
		"auto",		"break",	"case",
		"char",		"continue",	"default",
		"do",		"double",	"else",
		"extern",	"float",	"for",
		"goto",		"if",		"int",
		"long",		"register",	"return",
		"short",	"sizeof",	"static",
		"struct",	"switch",	"typedef",
		"union",	"unsigned",	"while",
		"entry",	
		"void",		"enum",
		"include",	"define",	"undef",
		"ifdef",	"ifndef",	"endif"
		};

static int	InitKeyword(keyword)
char		***keyword;
{
	int	k;

	if ((*keyword=(char **)calloc(DEFAULT_KEYWORD_NO,
				      sizeof(**keyword)))==NULL) return(0);
	for (k=0; k<DEFAULT_KEYWORD_NO; k++) (*keyword)[k]=defaultKeyword[k];
	return(DEFAULT_KEYWORD_NO);
}

static int	ReadKeyword(name,keyword)
char		*name;
char		***keyword;
{
	FILE	*fp;
	int	keywordno,k,l;
	char	kywdbuf[KEYWORD_LENGTH_MAX];

	free(*keyword);
	if ((fp=fopen(name,"r"))==NULL) return(0);
	fscanf(fp,"%d",&keywordno);
	if ((*keyword=(char **)calloc(keywordno,sizeof(**keyword)))==NULL) {
	    fclose(fp); return(0);
	}
	for (k=0; k<keywordno; k++) {
	    fscanf(fp,"%s",kywdbuf); l=strlen(kywdbuf)+1;
	    if (((*keyword)[k]=(char *)calloc(l,sizeof(***keyword)))==NULL) {
		for (k--; k>=0; k--) free((*keyword)[k]);
		fclose(fp); return(0);
	    }
	    strcpy((*keyword)[k],kywdbuf);
	}
	fclose(fp); return(keywordno);
}

static int	CheckKeyword(token,keywordno,keyword)
char		*token;
int		keywordno;
char		**keyword;
{
	for (; keywordno>0; keyword++, keywordno--)
	    if (!strcmp(token,*keyword)) return(1);
	return(0);
}

#define ANSI_REVERSE	1
#define SUN_REVERSE	2
#define BS_UNDERSCORE	3

#define DISPLAY_MODE	4
#define DISPLAY_UNIT	"0.25p"
#define DISPLAY_ROMAN	'R'
#define DISPLAY_BOLD	'B'
#define DISPLAY_ITALIC	'I'

#define DEFAULT_ENHANCE(token)	BSUnderscore(token)

static void	ANSIReverse(s)
char		*s;
{
	fprintf(stdout,"\033[33m%s\033[0m",s);
}

static void	SUNReverse(s)
char		*s;
{
	fprintf(stdout,"\033[1m%s\033[m",s);
}

static void	BSUnderscore(s)
char		*s;
{
	while (*s) fprintf(stdout,"_\010%c",*(s++));
}

static void	DisplayModePutC(c)
char		c;
{
	switch(c) {
	    case '\'' : fputs("\\'",stdout); break;
	    case '\\' : fputs("\\e",stdout); break;
	    case '.'  : fputs("\\.",stdout); break;
	    default   : fputc(c,stdout); break;
	}
}

static void	DisplayModePutS(s)
char		*s;
{
	while (*s) DisplayModePutC(*(s++));
}

#define STRING_LENGTH	256
#define EXPAND_PIPE	"expand"

static int	Lister(name,kywdno,kywd,enhance)
char		*name,**kywd;
int		kywdno;
int		enhance;
{
	char	pipe[STRING_LENGTH],token[KEYWORD_LENGTH_MAX];
	FILE	*fp;
	int	l,c,f;

	sprintf(pipe,"%s %s",EXPAND_PIPE,name);
	if ((fp=popen(pipe,"r"))==NULL) return(0);

	if (enhance>=DISPLAY_MODE) fputs(".LP\n.DS L\n.ft CW\n",stdout);

	while ((l=GetToken(fp,KEYWORD_LENGTH_MAX,token,CheckChar))>=0)
	    if (l)
		if (CheckKeyword(token,kywdno,kywd))
		    switch(enhance) {
			case ANSI_REVERSE :
			    ANSIReverse(token); break;
			case SUN_REVERSE :
			    SUNReverse(token); break;
			case BS_UNDERSCORE :
			    BSUnderscore(token); break;

	case DISPLAY_MODE :
	case DISPLAY_ROMAN :
	case DISPLAY_BOLD :
	case DISPLAY_ITALIC :
	    if (enhance!=DISPLAY_MODE) fprintf(stdout,"\\f%c",enhance);

	    DisplayModePutS(token); fputs("\\h\'-\\w\'",stdout);
	    DisplayModePutS(token); fputs("\'u\'",stdout);

	    if (enhance==DISPLAY_MODE)
		fprintf(stdout,"\\h\'+%s\'\\v\'+%s\'",DISPLAY_UNIT,
						      DISPLAY_UNIT);
	    else
		fputs("\\fP\\h\'+\\w\'",stdout);

	    DisplayModePutS(token);

	    if (enhance==DISPLAY_MODE)
		fprintf(stdout,"\\h\'-%s\'\\v\'-%s\'",DISPLAY_UNIT,
						      DISPLAY_UNIT);
	    else
		fputs("\'u\'",stdout);

	    break;

			default :
			    DEFAULT_ENHANCE(token);
		    }
		else

	if (enhance>=DISPLAY_MODE) DisplayModePutS(token); else

		    fputs(token,stdout);
	    else {

	if (enhance>=DISPLAY_MODE) DisplayModePutC(*token); else

		fputc(*token,stdout);
		switch(*token) {
		    case '/' :
			if ((c=fgetc(fp))!='*')
			    ungetc(c,fp);
			else {
			    fputc('*',stdout);
			    for (f=0; (c=fgetc(fp))!=EOF;) {

	if (enhance>=DISPLAY_MODE) DisplayModePutC(c); else

				fputc(c,stdout);
				if (f && c=='/') break;
				f=(c=='*');
			    }
			}
			break;
		    case '"' :
		    case '\'' :
			for (f=0; (c=fgetc(fp))!=EOF;) {

	if (enhance>=DISPLAY_MODE) DisplayModePutC(c); else

			    fputc(c,stdout);
			    if (f)
				f=0;
			    else {
				if (*token==c) break;
				if (c=='\\') f=1;
			    }
			}
			break;
		}
	    }

	if (enhance>=DISPLAY_MODE) fputs(".ft P\n.DE\n",stdout);

	pclose(fp); return(1);
}

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}

static void	Usage()
{
	Error("usage : lfc {-a|s|p|d|dR|dB|dI} {-k keywordFile} {files}");
}

static void	BadFile(name)
char		*name;
{
	fputs(name,stderr); Error(" : file not found.");
}

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	enhance,keyword,no,files=0;
	char	**kywd;

	if (!(no=InitKeyword(&kywd))) Error("memory is too little");

	enhance=keyword=0;
	while (--argc>0)
	    if (**(++argv)=='-')
		switch ((*argv)[1]) {
		    case 'a' : case 'A' :
			if (enhance) Usage(); enhance=ANSI_REVERSE; break;
		    case 's' : case 'S' :
			if (enhance) Usage(); enhance=SUN_REVERSE; break;
		    case 'p' : case 'P' :
			if (enhance) Usage(); enhance=BS_UNDERSCORE; break;
		    case 'd' : case 'D' :
			if (enhance) Usage();
			switch((*argv)[2]) {
			    case '\0' :
				enhance=DISPLAY_MODE; break;
			    case 'r' : case 'R' :
				enhance=DISPLAY_ROMAN; break;
			    case 'b' : case 'B' :
				enhance=DISPLAY_BOLD; break;
			    case 'i' : case 'I' :
				enhance=DISPLAY_ITALIC; break;
			    default :
				Usage();
			}
			break;
		    case 'k' : case 'K' :
			if (keyword || --argc==0) Usage();
			if (!(no=ReadKeyword(*(++argv),&kywd))) BadFile(*argv);
			keyword=1; break;
		    default :
			Usage();
		}
	    else {
		++files; if (!Lister(*argv,no,kywd,enhance)) BadFile(*argv);
	    }

	if (!files)
	    if (!Lister("",no,kywd,enhance)) Error("unknown lister error.");

	return(0);
}
