
#include <stdio.h>

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}

#define STRLEN	256

static int	Skip(pnm)
FILE		*pnm;
{
	int	chr;

	while ((chr=fgetc(pnm))==' ' || chr=='\t' || chr=='\r' || chr=='\n');

	if (chr==EOF) Error("unexpected end of file.");
	if (chr=='#') {
	    while ((chr=fgetc(pnm))!=EOF && chr!='\n'); return(Skip(pnm));
	}
	return(chr);
}

static int	Get01(pnm)
FILE		*pnm;
{
	int	chr=Skip(pnm);

	if (chr!='0' && chr!='1') Error("not a binary number character.");

	return(chr-'0');
}

static int	GetNum(pnm)
FILE		*pnm;
{
	int	chr=Skip(pnm),num=0;

	if (chr<'0' || chr>'9') Error("not a numeric character.");

	do num=num*10+chr-'0'; while ((chr=fgetc(pnm))>='0' && chr<='9');

	ungetc(chr,pnm); return(num);
}

#define FOR(LEN)		for (len=LEN; len>0; len--)
#define COPY123(LEN,get)	FOR(LEN) printf("%d\n",get(pnm))
#define COPY456(LEN)		FOR(LEN) putchar(fgetc(pnm))
#define SKIP123(LEN,get)	FOR(LEN) (void)get(pnm)
#define SKIP456(LEN)		SKIP123(LEN,fgetc)

int	main(argc,argv)
int	argc;
char	**argv;
{
	FILE	*pnm;
	int	page,magic,wid,hei,len;

	if (argc!=2 && argc!=3) Error("usage : pnmpage {PNM} page");

	if (argc==2)
	    pnm=stdin;
	else
	    if ((pnm=fopen(argv[1],"r"))==NULL) Error("PNM file not found.");

	for (page=atoi(argv[argc-1]); page>0; page--) {
	    if (Skip(pnm)!='P') Error("magic number not found.");

	    if ((magic=fgetc(pnm)-'0')<1 || magic>6) Error("bad magic number.");

	    if ((wid=GetNum(pnm))<=0 ||
		(hei=GetNum(pnm))<=0) Error("bad image size.");

	    if (page==1) printf("P%d\n%d\t%d",magic,wid,hei);

	    if (magic!=1 && magic!=4)
		if (page==1)
		    printf("\t%d\n",GetNum(pnm));
		else
		    (void)GetNum(pnm);
	    else
		if (page==1) putchar('\n');

	    if (magic==4 || magic==5 || magic==6) (void)fgetc(pnm);

	    if (page==1)
		switch(magic) {
		    case 1 : COPY123(hei*wid,  Get01 ); break;
		    case 2 : COPY123(hei*wid,  GetNum); break;
		    case 3 : COPY123(hei*wid*3,GetNum); break;
		    case 4 : COPY456(hei*((wid+7)/8)); break;
		    case 5 : COPY456(hei*wid        ); break;
		    case 6 : COPY456(hei*wid*3      ); break;
		}
	    else
		switch(magic) {
		    case 1 : SKIP123(hei*wid,  Get01 ); break;
		    case 2 : SKIP123(hei*wid,  GetNum); break;
		    case 3 : SKIP123(hei*wid*3,GetNum); break;
		    case 4 : SKIP456(hei*((wid+7)/8)); break;
		    case 5 : SKIP456(hei*wid        ); break;
		    case 6 : SKIP456(hei*wid*3      ); break;
		}
	}
	if (pnm!=stdin) fclose(pnm);
}
