
#include <stdio.h>

#ifndef ITALIC
#define ITALIC	"I"
#endif

#define ERROR(msg)	fputs(msg,stderr); fputc('\n',stderr)

static void	Error(msg)
char		*msg;
{
	ERROR(msg); exit(1);
}

#define HEADER	puts(".DS L"); puts(".ft CW"); printf(".ps %lg\n",ps); \
					       printf(".vs %lgp\n",vs)
#define TRAILER	puts(".ft P"); puts(".DE")

static void	ROFFDisplay(file,ps,vs,ll)
FILE		*file;
double		ps,vs;
int		ll;
{
	int	c,n=0,l=0;

	HEADER;

	while ((c=fgetc(file))!=EOF)
	    switch(c) {
		case '\f' :
		    if (l>0 || n>0) {
			if (n>0) { putchar('\n'); n=0; }

			TRAILER; l=0; HEADER;
		    }
		    break;
		case '\n' :
		    putchar('\n'); n=0;
		    if (++l==ll) { TRAILER; l=0; HEADER; }
		    break;
		case '\t' :
		    do {
			putchar('\\'); putchar(' ');
		    } while ((++n)&7);
		    break;
		case '_' :
		    if ((c=fgetc(file))!='\b') {
			ungetc(c,file); putchar('_');
		    }
		    else {
		        c=fgetc(file);
			printf("\\f%s%c\\h\'-\\w\'%c\'u\'\\fP ",ITALIC,c,c);
		    }
		    ++n; break;
		case '\\' :
		    putchar('\\'); putchar('e'); ++n; break;
		case '.' :
		case '\'' :
		    if (n>0) putchar(c); else printf("\\&%c",c);
		    ++n; break;
		default :
		    if (c&0x80) {
			putchar('\\'); putchar('f'); putchar('R');
			do {
			    putchar(c); putchar(fgetc(file)); n+=2;
			} while ((c=fgetc(file))!=EOF && c&0x80);
			putchar('\\'); putchar('f'); putchar('P');
			ungetc(c,file);
		    }
		    else {
			if (c==' ') putchar('\\'); putchar(c); ++n;
		    }
		    break;
	    }

	if (n>0) putchar('\n');

	TRAILER;
}

#define DEFAULT_PS	9.0
#define DEFAULT_VS	10.0

#define DEFAULT(msg,val)	\
	fputs(msg,stderr); fprintf(stderr," (default=%lg).\n",val)

static void	Usage()
{
	ERROR(  "usage : roffl {options} {files}");
	ERROR(  "\toptions");
	DEFAULT("\t\t-f size   : set font size in point",DEFAULT_PS);
	DEFAULT("\t\t-v space  : set vertical spacing in point",DEFAULT_VS);
	ERROR(  "\t\t-w width  : set horizontal line width in inch.");
	Error(  "\t\t-l number : set line number per page.");
}

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	arg,fFlag=0,vFlag=0,wFlag=0,lFlag=0,ll=0,files=0;
	double	ps=DEFAULT_PS,vs=DEFAULT_VS,lw=0.0;
	FILE	*file;

	for (arg=1; arg<argc; arg++)
	    if (argv[arg][0]!='-')
		++files;
	    else
		switch(argv[arg][1]) {
		    case 'f' : case 'F' :
			if (fFlag || ++arg==argc) Usage();
			sscanf(argv[arg],"%lf",&ps);
			if (ps<=0.0) Error("bad font size.");
			fFlag=1; break;
		    case 'v' : case 'V' :
			if (vFlag || ++arg==argc) Usage();
			sscanf(argv[arg],"%lf",&vs);
			if (vs<=0.0) Error("bad vertical spacing.");
			vFlag=1; break;
		    case 'w' : case 'W' :
			if (wFlag || ++arg==argc) Usage();
			sscanf(argv[arg],"%lf",&lw);
			if (lw<0.0) Error("bad horizontal line width.");
			wFlag=1; break;
		    case 'l' : case 'L' :
			if (lFlag || ++arg==argc) Usage();
			sscanf(argv[arg],"%d",&ll);
			if (ll<0) Error("bad line number per page.");
			lFlag=1; break;
		    default :
			Usage();
		}

	if (lw>0.0) printf(".nr LL %lgi\n",lw);

	puts(".ds CH");
/*
	puts(".ds RH %");
*/
	puts(".LP");

	if (files==0)
	    ROFFDisplay(stdin,ps,vs,ll);
	else
	    for (arg=1; arg<argc; arg++)
		if (argv[arg][0]=='-')
		    switch(argv[arg][1]) {
			case 'f' : case 'F' :
			case 'v' : case 'V' :
			case 'w' : case 'W' :
			case 'l' : case 'L' : ++arg; break;
		    }
		else
		    if ((file=fopen(argv[arg],"r"))==NULL)
			fprintf(stderr,"%s : file not found.\n",argv[arg]);
		    else {
			ROFFDisplay(file,ps,vs,ll); fclose(file);
		    }

	return 0;
}
