
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#define PATH_MAX	2048

static void	Error(msg,inf)
char		*msg,*inf;
{
	fprintf(stderr,"%s%s.\n",msg,inf); exit(1);
}

static void	GetAbsPath(path,absPath)
char		*path,*absPath;
{
	char	cwd[PATH_MAX];

	if (getcwd(cwd,PATH_MAX)==NULL || chdir(path)==(-1) ||
	    getcwd(absPath,PATH_MAX)==NULL || chdir(cwd)==(-1))
	    Error("error when reading absolute path of ",path);
}

typedef struct {
		char	*name;
		int	isDir;
	} File;

typedef struct {
		char	*name;
		int	number;
		File	*file;
	} Dir;

static void	FreeDir(dir)
Dir		*dir;
{
	int	index;

	free(dir->name);
	for (index=0; index<dir->number; index++)
	    free((dir->file)[index].name);
	free(dir->file);
}

static int	CompareDirFileName(file1,file2)
File		*file1,*file2;
{
	return(strcmp(file1->name,file2->name));
}

static void	GetDir(dir,path)
Dir		*dir;
char		*path;
{
	char		absPath[PATH_MAX];
	DIR		*d;
	struct dirent	*e;
	int		index;
	struct stat	s;
	char		*ewamf="error when allocating memory for ";

	GetAbsPath(path,absPath);
	if ((dir->name=(char *)malloc(strlen(absPath)+1))
	    ==NULL) Error(ewamf,path);

	strcpy(dir->name,absPath);
	if ((d=opendir(absPath))==NULL) Error("error when opening ",path);

	for (dir->number=0; (e=readdir(d))!=NULL; dir->number++) ;

	if ((dir->file=(File *)calloc(dir->number,sizeof(*(dir->file))))
	    ==NULL) Error(ewamf,path);

	rewinddir(d);

	for (index=0; (e=readdir(d))!=NULL; index++) {
	    if (((dir->file)[index].name=(char *)malloc(strlen(e->d_name)+1))
		==NULL) Error(ewamf,path);

	    strcpy((dir->file)[index].name,e->d_name);

	    sprintf(absPath,"%s/%s",dir->name,e->d_name);
	    if (stat(absPath,&s)
		==(-1)) Error("error when reading the status of ",path);

	    (dir->file)[index].isDir=(s.st_mode&S_IFMT)==S_IFDIR;
	}
	closedir(d);

	qsort(dir->file,dir->number,sizeof(*(dir->file)),CompareDirFileName);
}

typedef	struct {
		int	size,ptr;
		char	**name;
	} Cueue;

static void	MakeCueue(cueue,size)
Cueue		*cueue;
int		size;
{
	int	index;
	char	*makeCueueError="make cueue error";

	if (size<=0) Error("bad cueue size","");

	cueue->size=size; cueue->ptr=0;

	if ((cueue->name=(char **)calloc(cueue->size,sizeof(*(cueue->name))))
	    ==NULL) Error(makeCueueError,"");

	for (index=0; index<cueue->size; index++)
	    if (((cueue->name)[index]=(char *)malloc(1))
		==NULL) Error(makeCueueError,"");
	    else
		*(cueue->name)[index]='\0';
}

static void	ClearCueue(cueue)
Cueue		*cueue;
{
	int	index;

	cueue->ptr=0;
	for (index=0; index<cueue->size; index++)
	    if (*((cueue->name)[index])!='\0') {
		free((cueue->name)[index]);
		if (((cueue->name)[index]=(char *)malloc(1))
		    ==NULL) Error("clear cueue error","");
		else
		    *(cueue->name)[index]='\0';
	    }
}

static void	PutCueue(cueue,name)
Cueue		*cueue;
char		*name;
{
	free((cueue->name)[cueue->ptr]);
	if (((cueue->name)[cueue->ptr]=
	    (char *)malloc(strlen(name)+1))
	    ==NULL) Error("put cueue error","");
	else
	    strcpy((cueue->name)[cueue->ptr],name);
	if (++(cueue->ptr)==cueue->size) cueue->ptr=0;
}

static char	*GetCueue(cueue,index)
Cueue		*cueue;
int		index;
{
	if ((index+=cueue->ptr)>=cueue->size) index-=cueue->size;
	return((cueue->name)[index]);
}

static void	PutDirFile(dir,file,cueue)
char		*dir,*file;
Cueue		*cueue;
{
	char	*name;

	if ((name=(char *)malloc(strlen(dir)+strlen(file)+2))
	    ==NULL) Error("error when putting directory and file name","");
	else
	    sprintf(name,"%s/%s",dir,file);

	PutCueue(cueue,name);
	free(name);
}

static void	RunProgram(name,cueue)
char		*name;
Cueue		*cueue;
{
	int	length,index;
	char	*execute;

	length=strlen(name)+1;
	for (index=0; index<cueue->size; index++)
	    length+=(strlen(GetCueue(cueue,index))+1);

	if ((execute=(char *)malloc(length))
	    ==NULL) Error("memory error when running program ",name);

	strcpy(execute,name);
	for (index=0; index<cueue->size; index++) {
	    strcat(execute," "); strcat(execute,GetCueue(cueue,index));
	}

	fprintf(stderr,"running program: %s\n",execute);
	system(execute);
	free(execute);
}

static void	PrintCueue(execute,cueue)
char		*execute;
Cueue		*cueue;
{
	int	count,index;
	char	*name;

	fprintf(stderr,"argument of execute program: %s\n",execute);
	count=0;
	for (index=0; index<cueue->size; index++)
	    if (strlen(name=GetCueue(cueue,index))!=0)
		fprintf(stderr,"\t%4d/%d: %s\n",++count,cueue->size,name);

	if (count==0) fputs("\tnothing.\n",stderr);
}

#define SCREEN_WIDTH	79
#define MENU_NO		5
#define MENU_WIDTH	11

static char	*menu[MENU_NO]={"   !: input",
				"   r: run",
				"   p: print",
				"   c: clear",
				"   q: quit"};

static void	PrintString(width,string)
int		width;
char		*string;
{
	while (--width>=0) fputc((*string)?*(string++):' ',stderr);
}

static void	PrintNameAndMenu(dir)
Dir		*dir;
{
	int	width,index,length;
	File	*file;
	char	name[PATH_MAX];

	fprintf(stderr,"\npath: %s\n",dir->name);

	width=0;
	for (file=dir->file, index=0; index<dir->number; index++, file++) {
	    length=strlen(file->name)+7;
	    if (width<length) width=length;
	}
	if (width<MENU_WIDTH) width=MENU_WIDTH;
	width+=2;

	length=0;
	for (file=dir->file, index=1; index<=dir->number; index++, file++) {
	    if ((length+=width)>SCREEN_WIDTH) {
		length=width; fputc('\n',stderr);
	    }
	    sprintf(name,"%4d: %s%c",
			 index,file->name,(file->isDir)?'/':' ');
	    PrintString(width,name);
	}
	fputc('\n',stderr);

	length=0;
	for (index=0; index<MENU_NO; index++) {
	    if ((length+=width)>SCREEN_WIDTH) {
		length=width; fputc('\n',stderr);
	    }
	    PrintString(width,menu[index]);
	}
	fputc('\n',stderr);
}

int	main(argc,argv)
int	argc;
char	**argv;
{
	char	*execute,path[PATH_MAX],command[PATH_MAX];
	Cueue	cueue;
	Dir	dir;
	int	index,refresh;

	if (argc!=3 && argc!=4)
	    Error("usage : slector execute number {path}","");

	execute=argv[1];
	MakeCueue(&cueue,atoi(argv[2]));
	strcpy(path,(argc==3)?".":argv[3]);

	for (;;) {
	    GetDir(&dir,path);

	    refresh=1;
	    for (;;) {
		if (refresh) PrintNameAndMenu(&dir);

		fputs("select above = ",stderr); gets(command);
		switch(*command) {
		    case '!' :
			PutCueue(&cueue,&(command[1]));
			refresh=0; break;
		    case 'r' :
		    case 'R' :
			RunProgram(execute,&cueue);
			refresh=1; break;
		    case 'p' :
		    case 'P' :
			PrintCueue(execute,&cueue);
			refresh=1; break;
		    case 'c' :
		    case 'C' :
			ClearCueue(&cueue);
			refresh=0; break;
		    case 'q' :
		    case 'Q' :
			exit(0); break;
		    default :
			index=atoi(command);
			if (index>=1 && index<=dir.number)
			    if (dir.file[index-1].isDir)
				goto foundDir;
			    else
				PutDirFile(dir.name,
					   dir.file[index-1].name,
					   &cueue);
			refresh=0; break;
		}
	    }
foundDir:;
	    sprintf(path,"%s/%s",dir.name,dir.file[index-1].name);
	    FreeDir(&dir);
	}
	return(0);
}
