
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef	GRIDS
#define GRIDS	18
#endif

#ifndef	LEVELS
#define LEVELS	12
#endif

#ifndef	M_PI
#define M_PI	3.14159265358979323846
#endif

static void	Error(char *msg)
{
	(void)fputs(msg,stderr); (void)fputc('\n',stderr); exit(1);
}

#define LEN	2048
#define EPS	1e-9
#define INF	1e+37

int	main(int argc,char **argv)
{
	double	x,y,z,h,v,d,
		HX,VX,DX,HY,VY,DY,HZ,VZ,DZ,x0,y0,z0,h0,v0,d0,
		r0,hh,vv,dd,hx,vx,dx,hy,vy,dy,hz,vz,dz,hv,vd,dh,
		A,B,C,R,s,a0,b0,c0,c,cosC,sinC,b,cosB,sinB,a,cosA,sinA,
		xh,xv,xd,yh,yv,yd,zh,zv,zd,r;
	char	*f,T[LEN];
	FILE	*F;
	int	N,m,l,k,j,i;

	struct Marker {
		double	x,y,z,h,v,d;
	} *M;

	switch(argc) {
	    case 3:
	    case 4:
		x=y=z=1.0; h=v=d=atof(argv[1]); f=argv[2]; break;
	    case 6:
	    case 7:
		x=y=z=atof(argv[1]); h=atof(argv[2]);
				     v=atof(argv[3]);
				     d=atof(argv[4]); f=argv[5]; break;
	    case 8:
	    case 9:
		x=atof(argv[1]); h=atof(argv[4]);
		y=atof(argv[2]); v=atof(argv[5]);
		z=atof(argv[3]); d=atof(argv[6]); f=argv[7]; break;
	    default:
    Error("usage : check_marker ratio marker.txt {residual.txt}\n"
	  "        check_marker Uxyz Uh Uv Ud marker.txt {residual.txt}\n"
	  "        check_marker Ux Uy Uz Uh Uv Ud marker.txt {residual.txt}");
	}
	if (f!=argv[2] && (x<EPS || y<EPS || z<EPS)) Error("too small unit.");

	if ((HX=h/x)<1.0 || (VX=v/x)<1.0 || (DX=d/x)<1.0 ||
	    (HY=h/y)<1.0 || (VY=v/y)<1.0 || (DY=d/y)<1.0 ||
	    (HZ=h/z)<1.0 || (VZ=v/z)<1.0 || (DZ=d/z)<1.0)
	    (void)fputs("too small ratio (warning).\n",stderr);

	if (f[0]=='-')
	    F=stdin;
	else
	    if ((F=fopen(f,"r"))==NULL) Error("file not found.");

	N=0; M=NULL; x0=y0=z0=h0=v0=d0=0.0;
	while (fgets(T,LEN,F)!=NULL)
	    if (sscanf(T,"%lf %lf %lf %lf %lf %lf",&x,&y,&z,&h,&v,&d)==6) {
		m=N++;
		if ((M=(struct Marker *)realloc(M,sizeof(*M)*N))==NULL)
		    Error("memory allocation error.");

#define X	M[m].x
#define Y	M[m].y
#define Z	M[m].z
#define H	M[m].h
#define V	M[m].v
#define D	M[m].d

		x0+=(X=x+0.5); y0+=(Y=y+0.5); z0+=(Z=z+0.5);
		h0+=(H=h+0.5); v0+=(V=v+0.5); d0+=(D=d+0.5);
	    }
#ifdef	DEBUG
	    else
		(void)fputs(T,stderr);
#endif
	if (N<4) Error("too few markers.");

	if (F!=stdin) (void)fclose(F);

	x0/=(double)N; y0/=(double)N; z0/=(double)N;
	h0/=(double)N; v0/=(double)N; d0/=(double)N;

	r0=hh=vv=dd=hx=vx=dx=hy=vy=dy=hz=vz=dz=hv=vd=dh=0.0;
	for (m=0; m<N; m++) {
	    x=(X-=x0); y=(Y-=y0); z=(Z-=z0);
	    h=(H-=h0); v=(V-=v0); d=(D-=d0);

	    r0+=(x*x+y*y+z*z); hh+=(h*h); vv+=(v*v); dd+=(d*d);
			       hx+=(h*x); vx+=(v*x); dx+=(d*x);
			       hy+=(h*y); vy+=(v*y); dy+=(d*y);
			       hz+=(h*z); vz+=(v*z); dz+=(d*z); hv+=(h*v);
							        vd+=(v*d);
								dh+=(d*h);
	}
	A=B=C=M_PI; R=INF;
	for (s=M_PI/(double)GRIDS, l=0; l<LEVELS; l++, s/=(double)GRIDS) {
	    a0=A; b0=B; c0=C;
	    for (k=(-GRIDS); k<=GRIDS; k++) {
		c=c0+s*(double)k; cosC=cos(c); sinC=sin(c);
		for (j=(-GRIDS); j<=GRIDS; j++) {
		    b=b0+s*(double)j; cosB=cos(b); sinB=sin(b);
		    for (i=(-GRIDS); i<=GRIDS; i++) {
			a=a0+s*(double)i; cosA=cos(a); sinA=sin(a);

#define XH	(      cosB*cosC          )*HX
#define XV	(     -cosB*sinC          )*VX
#define XD	(      sinB               )*DX
#define YH	( sinA*sinB*cosC+cosA*sinC)*HY
#define YV	(-sinA*sinB*sinC+cosA*cosC)*VY
#define YD	(-sinA*cosB               )*DY
#define ZH	(-cosA*sinB*cosC+sinA*sinC)*HZ
#define ZV	( cosA*sinB*sinC+sinA*cosC)*VZ
#define ZD	( cosA*cosB               )*DZ

			xh=XH; xv=XV; xd=XD;
			yh=YH; yv=YV; yd=YD;
			zh=ZH; zv=ZV; zd=ZD;
		        if ((r=r0+(xh*xh+yh*yh+zh*zh)*hh
				 +(xv*xv+yv*yv+zv*zv)*vv
				 +(xd*xd+yd*yd+zd*zd)*dd
				 -2.0*( xh*hx+xv*vx+xd*dx
				       +yh*hy+yv*vy+yd*dy
				       +zh*hz+zv*vz+zd*dz
				       -(xh*xv+yh*yv+zh*zv)*hv
				       -(xv*xd+yv*yd+zv*zd)*vd
				       -(xd*xh+yd*yh+zd*zh)*dh))<R) {
			    A=a; B=b; C=c; R=r;
			}
		    }
		}
	    }
	}
	cosA=cos(A); sinA=sin(A);
	cosB=cos(B); sinB=sin(B);
	cosC=cos(C); sinC=sin(C); xh=XH; xv=XV; xd=XD;
				  yh=YH; yv=YV; yd=YD;
				  zh=ZH; zv=ZV; zd=ZD;

	(void)printf("%lg\t%lg\t%lg\n"
		     "%lf\t%lf\t%lf\n"
		     "%lf\t%lf\t%lf\n"
		     "%lf\t%lf\t%lf\n"
		     "%lg\t%lg\t%lg\n"
		     "%lf\n",
		     fmod(A*180.0/M_PI+180.0,360.0)-180.0,
		     fmod(B*180.0/M_PI+180.0,360.0)-180.0,
		     fmod(C*180.0/M_PI+180.0,360.0)-180.0,
		     xh,xv,xd,
		     yh,yv,yd,
		     zh,zv,zd,
		     x0-xh*h0-xv*v0-xd*d0,
		     y0-yh*h0-yv*v0-yd*d0,
		     z0-zh*h0-zv*v0-zd*d0,
		     (R<=0.0)?0.0:sqrt(R/(double)N));

	if (argc==4 || argc==7 || argc==9) {
	    if (argv[argc-1][0]=='-')
		F=stdout;
	    else
		if ((F=fopen(argv[argc-1],"w"))==NULL) Error("file not open.");

	    for (m=0; m<N; m++) {
		x=X-xh*H-xv*V-xd*D;
		y=Y-yh*H-yv*V-yd*D;
		z=Z-zh*H-zv*V-zd*D;
		(void)fprintf(F,"%lg %lg %lg\t%lg %lg %lg\t%lf"
#ifdef	RXYZ
				"\t%lg %lg %lg"
#endif
				"\n",
				X+x0-0.5,Y+y0-0.5,Z+z0-0.5,
				H+h0-0.5,V+v0-0.5,D+d0-0.5,
				sqrt(x*x+y*y+z*z)
#ifdef	RXYZ
				,x,y,z
#endif
			     );
	    }
	    if (F!=stdout) (void)fclose(F);
	}
	free(M);

	return 0;
}
