#!/bin/csh
# usage : hg_mm.csh hg.csv {min_CTV min_Pels max_CTV max_Pels}
#
set xl='CT-value (estimated LAC, 1/cm)'
set x1=-
set x2=-
#
set yl='log(pixels)'
set y1=-
set y2=-
#
set lpj=(graph 0.9,0.8 right)	# label position and justification
#
if ($#argv != 1 && $#argv != 5) then
  stderr usage : hg_mm.csh hg.csv \{min_CTV min_Pels max_CTV max_Pels\} ; exit 1
endif
#
set csv=$argv[1]
if (!(-e $csv)) then
  stderr $csv : file not found. ; exit 1
endif
#
if ($#argv == 5) then
  set x1=$argv[2] ; set y1=$argv[3] ; set x2=$argv[4] ; set y2=$argv[5]
endif
if ($x1 == -) set x1=`head -1 $csv | awk -F, '{ printf "%f\n",($2+$3)/2 }'`
if ($y1 == -) set y1=`awk -F, 'NR==1 || $4<y { y=$4 } \
                               END { printf (y>=10^10)?"%g\n":"%f\n",y }' $csv`
if ($x2 == -) set x2=`tail -1 $csv | awk -F, '{ printf "%f\n",($2+$3)/2 }'`
if ($y2 == -) set y2=`awk -F, 'NR==1 || $4>y { y=$4 } \
                               END { printf (y>=10^10)?"%g\n":"%f\n",y }' $csv`
#
set label=`realpath $csv | awk -F/ '{ print $(NF-1) }'`
#
set z=(`( echo -n ${x1},${x2}, ; head -1 $csv ) | \
        awk -F, '{ m=($4+$5)/2; d=$5-$4; print ($1-m)/d+$3,($2-m)/d+$3 }'`)
#
gnuplot -persist <<fin
  set terminal x11
  unset key
  set grid
  set logscale y
  set xtics nomirror
  set x2tics
  set xlabel '${xl}'
  set ylabel '${yl}'
  set xrange [${x1}:${x2}]
  set yrange [${y1}:${y2}]
  set x2range [${z[1]}:${z[2]}]
  set label '${label}' at $lpj
  plot "< cat $csv | tr , ' '" using ((\$2+\$3)/2):4 with lines linewidth 2
fin
