#!/bin/csh
# usage : tg.csh {z1 z2} {DO}
#
#set hp2tg=hp2tg	# CBP by both CUDA and single thread
#set hp2tg=hp2tg_wai	# CUDA with angular interpolation
#set hp2tg=hp2tg_nai	# single thread without angular interpolation
#set hp2tg=hp2tg_int	# single thread with BP by integer arithmetic
set hp2tg=hp2tg_t	# both CUDA and multi-thread CBP
#set hp2tg=hp2tg_twai	# CUDA with angular interpolation
#set hp2tg=hp2tg_tnai	# multi-thread without angular interpolation
#set hp2tg=hp2tg_tint	# multi-thread with BP by integer arithmetic
#
setenv CUDA_GPU 0	# device number of CUDA-GPU
#setenv CBP_THREADS 4	# number of threads
#
switch($#argv)
  case 0 :
    set z12='' ; set ab='' ; breaksw
  case 1 :
    set z12='' ; set ab=(0 $argv[1]) ; breaksw
  case 2 :
    set z12=($argv[1-2]) ; set ab='' ; breaksw
  case 3 :
    set z12=($argv[1-2]) ; set ab=(0 $argv[3]) ; breaksw
  default:
    stderr 'usage : tg.csh {z1 z2} {DO}' ; exit 1
endsw
#
if (!(-e raw)) then
  stderr raw : directory not found. ; exit 1
endif
#
if (!(-e DO.tbl)) then
  stderr DO.tbl : file not found. ; exit 1
endif
#
if ($?PS_CM == 0) then
  stderr 'PS_CM : environmental variable for pixel size (cm) not set.' ; exit 1
endif
#
if (-e tg) then
  stderr tg : directory already exists. ; exit 1
endif
#
set nrz=(`head -1 DO.tbl`)
if ($#z12 != 2) set z12=(0 `expr $nrz[2] - 1`)
if ($#ab != 2) set ab=(`tail -1 DO.tbl | cut -f1-2`)
#
set awk=('NR==1 { z1=z2=$1; f1=$3; f2=$4 }' \
         'NR>=2 { if ($1<z1) z1=$1; if ($1>z2) z2=$1;' \
                 'if ($3<f1) f1=$3; if ($4>f2) f2=$4 }' \
         'END { printf "%s\t%s\t%s\t%s\t%s\t%s\n",' \
                       $PS_CM ',' $nrz[1] ',z1,z2,f1,f2 }')
mkdir tg
dit_tbl raw | $hp2tg $z12 $PS_CM $ab[2] $ab[1] 0 16 tg | tee tg.tbl | \
awk "$awk" | tee tg.log
